#include <winsock.h>
#include <stdio.h>
#include "isock.h"

/**************************************************************************
 *Name: InitializeSocket(char*)											  *
 *Description:  Do version checking with winsock.dll, initialize the      *
 *				socket and resolve local host information				  *
 *Possible returns:  0 for success										  *
 *					 SOCKET_ERROR (-1) for failure						  *
 **************************************************************************/
int CISock::InitializeSocket(char* host_name)
{
	int err; 
	SOCKADDR_IN s_in;

	err = WSAStartup(MAKEWORD(1,1), &m_pwsaData); 
 
	if (err != 0) 
		// Tell the user that we couldn't find a useable  
		// winsock.dll.      
		return err; 
 
	// Confirm that the Windows Sockets DLL supports 1.1. 
	// Note that if the DLL supports versions greater  
	// than 1.1 in addition to 1.1, it will still return 
	// 1.1 in wVersion since that is the version we  
	// requested. */ 
 
	if ( LOBYTE( m_pwsaData.wVersion ) != 1 || 
			HIBYTE( m_pwsaData.wVersion ) != 1 ) { 
		// Tell the user that we couldn't find a useable  
		// winsock.dll.  
		WSACleanup(); 
		return 0; 
	} 
	// The Windows Sockets DLL is acceptable. Proceed. 

	// Create the socket 
	m_pSocket = socket(PF_INET, SOCK_STREAM, 0);

	//Let's connect to the site 
	s_in.sin_family = AF_INET;
	s_in.sin_port = htons(25);
	s_in.sin_addr.S_un.S_addr = GetAddr(host_name);
	
	//Get local host information
	//if err returns anything other than 0, then there was an error
	if (gethostname(m_pLocalHostName, 256) == SOCKET_ERROR)
		return SOCKET_ERROR;
	
	//Make the socket connection
	err = connect(m_pSocket, (SOCKADDR*)&s_in, sizeof(s_in));
		
	
	//Return the error code from the connect function.  Will either return 0 or SOCKET_ERROR (-1)
	return err;
}

/**************************************************************************
 *Name: SendData(char*)													  *
 *Description:  Send the data through the connected socket.               *
 *Possible returns:  0 for success										  *
 *					 SOCKET_ERROR (-1) for failure						  *
 **************************************************************************/
int CISock::SendData(char* datatosend)
{
	int err;

	if ( (err = send(m_pSocket, datatosend, strlen(datatosend), 0)) == SOCKET_ERROR)
		//error occured when sendint
		return 0;
	return 0;
}

/**************************************************************************
 *Name: ReceiveData()													  *
 *Description:  Receives input from the socket.                           *
 *Possible returns:  Response code from socket read						  *
 *					 SOCKET_ERROR (-1) for failure						  *
 *Also returns the text error from the mail server						  *
 **************************************************************************/
int CISock::ReceiveData(char* returntext)
{
	char* pData= new char[500];
	char* token = NULL;
	int num_of_chars_returned = 0;
	
	//Keep receiving from the socket until we get a 
	//<cr><lf> are at the end.
	do
	{
		if( (num_of_chars_returned = recv(m_pSocket, pData, 499, 0)) == SOCKET_ERROR)
			return SOCKET_ERROR;
		pData[num_of_chars_returned]= '\0';
		strcat(returntext, pData);

	}while ( (pData[num_of_chars_returned - 2] != '\r') && (pData[num_of_chars_returned - 1] != '\n') );

	//Return whatever code that was generated by the read
	token = strtok(pData, " -");
	//Just go to the end so I can get the text.
	return (int)atoi(token);

}

// The following code is borrowed from the sample code at http://www.sockets.com
/*-----------------------------------------------------------
 * Function: GetAddr()
 *
 * Description: Given a string, it will return an IP address.
 *   - first it tries to convert the string directly
 *   - if that fails, it tries o resolve it as a hostname
 *
 * WARNING: gethostbyname() is a blocking function
 */
unsigned long CISock::GetAddr (LPSTR szHost) 
{
  LPHOSTENT lpstHost;
  u_long lAddr = INADDR_ANY;
  
  /* check that we have a string */
  if (*szHost) {
  
    /* check for a dotted-IP address string */
    lAddr = inet_addr (szHost);
  
    /* If not an address, then try to resolve it as a hostname */
    if ((lAddr == INADDR_NONE) &&
        (strcmp(szHost, "255.255.255.255"))) {
      
      lpstHost = gethostbyname(szHost);
      if (lpstHost) {  /* success */
        lAddr = *((u_long FAR *) (lpstHost->h_addr));
      } else {  
        lAddr = INADDR_ANY;   /* failure */
      }
    }
  }
  return (lAddr); 
} /* end GetAddr() */

